<?php defined('MW_PATH') || exit('No direct script access allowed');

/**
 * SupportTicketsExtModel
 * 
 * @package MailWizz EMA
 * @subpackage Support tickets
 * @author Serban George Cristian <cristian.serban@mailwizz.com> 
 * @link http://www.mailwizz.com/
 * @copyright 2013-2014 MailWizz EMA (http://www.mailwizz.com)
 * @license http://www.mailwizz.com/license/
 */
 
/**
 * This is the model class for table "support_ticket".
 *
 * The followings are the available columns in table 'support_ticket':
 * @property integer $ticket_id
 * @property integer $user_id
 * @property integer $customer_id
 * @property integer $department_id
 * @property integer $priority_id
 * @property string $subject
 * @property string $content
 * @property integer $replies_count
 * @property integer $rating
 * @property string $ip_address
 * @property string $status
 * @property string $date_added
 * @property string $last_updated
 *
 * The followings are the available model relations:
 * @property SupportTicketPriority $priority
 * @property SupportTicketDepartment $department
 * @property User $user
 * @property Customer $customer
 * @property SupportTicketReply[] $replies
 * @property SupportTicketReply[] $repliesCount
 */
class SupportTicket extends ActiveRecord
{
    const STATUS_OPEN = 'open';
    
    const STATUS_WAITING_STAFF_REPLY = 'waiting staff reply';
    
    const STATUS_WAITING_CUSTOMER_REPLY = 'waiting customer reply';
    
    const STATUS_CLOSED = 'closed';
    
	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return '{{support_ticket}}';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		$rules = array(
			array('customer_id, subject, content, status', 'required'),
			array('user_id, customer_id, department_id, priority_id', 'numerical', 'integerOnly'=>true),
			array('subject', 'length', 'max' => 255),
			array('status', 'length', 'max' => 30),
            array('customer_id', 'exist', 'className' => 'Customer', 'criteria' => array('condition' => '`status` = :st', 'params' => array(':st' => Customer::STATUS_ACTIVE))),
            array('user_id', 'exist', 'className' => 'User', 'criteria' => array('condition' => '`status` = :st', 'params' => array(':st' => User::STATUS_ACTIVE))),
            array('department_id', 'exist', 'className' => 'SupportTicketDepartment'),
            array('priority_id', 'exist', 'className' => 'SupportTicketPriority'),
            array('status', 'in', 'range' => array_keys($this->getStatusesList())),
            
			// The following rule is used by search().
			array('user_id, customer_id, department_id, priority_id, subject, content, status', 'safe', 'on'=>'search'),
		);
        
        return CMap::mergeArray($rules, parent::rules());
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		$relations = array(
			'priority'     => array(self::BELONGS_TO, 'SupportTicketPriority', 'priority_id'),
			'department'   => array(self::BELONGS_TO, 'SupportTicketDepartment', 'department_id'),
			'user'         => array(self::BELONGS_TO, 'User', 'user_id'),
			'customer'     => array(self::BELONGS_TO, 'Customer', 'customer_id'),
			'replies'      => array(self::HAS_MANY, 'SupportTicketReply', 'ticket_id', 'order' => 'reply_id DESC'),
            'repliesCount' => array(self::STAT, 'SupportTicketReply', 'ticket_id'),
		);
        
        return CMap::mergeArray($relations, parent::relations());
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		$labels = array(
			'ticket_id'     => Yii::t('ext_support_tickets', 'Ticket'),
			'user_id'       => Yii::t('ext_support_tickets', 'Assigned to'),
			'customer_id'   => Yii::t('ext_support_tickets', 'Customer'),
			'department_id' => Yii::t('ext_support_tickets', 'Department'),
			'priority_id'   => Yii::t('ext_support_tickets', 'Priority'),
			'subject'       => Yii::t('ext_support_tickets', 'Subject'),
			'content'       => Yii::t('ext_support_tickets', 'Content'),
            'replies_count' => Yii::t('ext_support_tickets', 'Replies'),
            'rating'        => Yii::t('ext_support_tickets', 'Rating'),
            'ip_address'    => Yii::t('ext_support_tickets', 'Ip address'),
		);
        
        return CMap::mergeArray($labels, parent::attributeLabels());
	}
    
    /**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributePlaceholders()
	{
		$placeholders = array(
			'customer_id'    => Yii::t('ext_support_tickets', 'Customer (autocomplete enabled)'),
		);
        
        return CMap::mergeArray($placeholders, parent::attributePlaceholders());
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 *
	 * Typical usecase:
	 * - Initialize the model fields with values from filter form.
	 * - Execute this method to get CActiveDataProvider instance which will filter
	 * models according to data in model fields.
	 * - Pass data provider to CGridView, CListView or any similar widget.
	 *
	 * @return CActiveDataProvider the data provider that can return the models
	 * based on the search/filter conditions.
	 */
	public function search()
	{
		$criteria = new CDbCriteria;
        $criteria->with = array();
        
        if (!empty($this->user_id)) {
            if (is_numeric($this->user_id)) {
                $criteria->compare('t.user_id', (int)$this->user_id);
            } else {
                $criteria->with['user'] = array(
                    'select'    => false,
                    'joinType'  => 'INNER JOIN',
                    'together'  => true,
                    'condition' => '(user.email LIKE :user OR user.first_name LIKE :user OR user.last_name LIKE :user)',
                    'params'    => array(':user' => '%' . $this->user_id . '%'),
                );
            }
        }
        
        if (!empty($this->customer_id)) {
            if (is_numeric($this->customer_id)) {
                $criteria->compare('t.customer_id', (int)$this->customer_id);
            } else {
                $criteria->with['customer'] = array(
                    'select'    => false,
                    'joinType'  => 'INNER JOIN',
                    'together'  => true,
                    'condition' => '(customer.email LIKE :customer OR customer.first_name LIKE :customer OR customer.last_name LIKE :customer)',
                    'params'    => array(':customer' => '%' . $this->customer_id . '%'),
                );
            }
        }
        
        if (!empty($this->department_id)) {
            if (is_numeric($this->department_id)) {
                $criteria->compare('t.department_id', (int)$this->department_id);
            } else {
                $criteria->with['department'] = array(
                    'select'    => false,
                    'joinType'  => 'INNER JOIN',
                    'together'  => true,
                    'condition' => 'department.name LIKE :department',
                    'params'    => array(':department' => '%' . $this->department_id . '%'),
                );
            }
        }
        
        if (!empty($this->priority_id)) {
            if (is_numeric($this->priority_id)) {
                $criteria->compare('t.priority_id', (int)$this->priority_id);
            } else {
                $criteria->with['priority'] = array(
                    'select'    => false,
                    'joinType'  => 'INNER JOIN',
                    'together'  => true,
                    'condition' => 'priority.name LIKE :priority',
                    'params'    => array(':priority' => '%' . $this->priority_id . '%'),
                );
            }
        }

		$criteria->compare('t.subject', $this->subject, true);
		$criteria->compare('t.status', $this->status);
        
        $criteria->order = 't.last_updated DESC';
        
		return new CActiveDataProvider(get_class($this), array(
            'criteria'     => $criteria,
            'pagination'   => array(
                'pageSize' => $this->paginationOptions->getPageSize(),
                'pageVar'  => 'page',
            ),
            'sort'  => array(
                'defaultOrder' => array(
                    't.ticket_id' => CSort::SORT_DESC,
                ),
            ),
        ));
	}

	/**
	 * Returns the static model of the specified AR class.
	 * Please note that you should have this exact method in all your CActiveRecord descendants!
	 * @param string $className active record class name.
	 * @return SupportTicket the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}
    
    protected function beforeSave()
    {
        $this->replies_count = $this->repliesCount;
        $this->rating        = $this->getRepliesAverageRating();
        return parent::beforeSave();
    }
    
    public function getStatusesList()
    {
        return array(
            self::STATUS_OPEN                   => Yii::t('ext_support_tickets', ucfirst(self::STATUS_OPEN)),
            self::STATUS_WAITING_STAFF_REPLY    => Yii::t('ext_support_tickets', ucfirst(self::STATUS_WAITING_STAFF_REPLY)),
            self::STATUS_WAITING_CUSTOMER_REPLY => Yii::t('ext_support_tickets', ucfirst(self::STATUS_WAITING_CUSTOMER_REPLY)),
            self::STATUS_CLOSED                 => Yii::t('ext_support_tickets', ucfirst(self::STATUS_CLOSED)),
        );
    }
    
    public function getDepartmentsList()
    {
        static $departments;
        if ($departments !== null) {
            return $departments;
        }
        $departments = array();
        $models = SupportTicketDepartment::model()->findAll();
        foreach ($models as $model) {
            $departments[$model->department_id] = $model->name;
        }
        return $departments;
    }
    
    public function getPrioritiesList()
    {
        static $priorities;
        if ($priorities !== null) {
            return $priorities;
        }
        $priorities = array();
        $models = SupportTicketPriority::model()->findAll();
        foreach ($models as $model) {
            $priorities[$model->priority_id] = $model->name;
        }
        return $priorities;
    }
    
    public function getStatusName($status = null)
    {
        if ($status === null) {
            $status = $this->status;
        }
        return Yii::t('ext_support_tickets', ucfirst($status));
    }
    
    public function getRepliesAverageRating()
    {
        if ($this->isNewRecord) {
            return 0;
        }
        $row = Yii::app()->getDb()->createCommand('SELECT AVG(rating) as rating FROM {{support_ticket_reply}} WHERE ticket_id = :tid AND rating > 0')->queryRow(true, array(':tid' => $this->ticket_id));
        return round($row['rating'], 2);
    }
}
